/***************************************************************************************************
*            Sample for the usage of class TopoART_R (approximation of a sine function)            *
*                           Created by Marko Tscherepanow, 11 February 2012                        *
***************************************************************************************************/

// Compile: mcs /r:LibTopoART.dll TopoART_R_sample1.cs
// Run: mono TopoART_R_sample1.exe

using System;
using System.IO;
using System.Text.RegularExpressions;
using LibTopoART;
using System.Globalization;

namespace LibTopoART_samples
{
	/// <summary>
	/// Function approximation using TopoART-R.
	/// <para>
	/// This sample train a TopoART-R network with 100 points sampled from a sine function.
	/// Then, sine values are predicted for 25 random values.
	/// </para>
	/// </summary>
	class TopoART_R_sample1
	{
		static void Main() {
			long i;
			
			// Input and output dimensions
			long i_dim=1;
			long d_dim=1;

			// Training data
			long training_sample_num=100; 
			decimal[][] i_values=new decimal[training_sample_num][];
			decimal[][] d_values=new decimal[training_sample_num][];
					
			// Create training samples (sine function)
			for(i=0;i<training_sample_num;i++) {
				i_values[i]=new decimal[i_dim];
				i_values[i][0]=(decimal)i*(decimal)Math.PI*2.0m/(decimal)(training_sample_num-1);
				d_values[i]=new decimal[d_dim];
				d_values[i][0]=((decimal)Math.Sin((double)i_values[i][0])+1.0m)/2.0m; 	//normalise d_vec
				i_values[i][0]/=((decimal)Math.PI*2.0m);		// normalise i_vec
				
				//Console.WriteLine("create training sample {0} -> {1}",i_values[i][0],d_values[i][0]);
			}
			
			// Create TopoART-R network using rho_a=0.99 and default values for the remaining parameters
			ITopoART_R tar=new TopoART_R(1,1,2,0.99m);
			
			// Train TopoART-R network
			Console.WriteLine("");
			Console.WriteLine("train TopoART-R network with {0} samples",training_sample_num);
			for(i=0;i<training_sample_num;i++) {
				tar.Learn(i_values[i],d_values[i]);
			}
			
			// Prediction
			decimal[] random_i_vec=new decimal[i_dim];
			TopoART_R_Prediction prediction;
			Random r = new Random();
			bool[] m_i_vec=new bool[1];
			m_i_vec[0]=false;				// variable is presented
			Console.WriteLine("");
			Console.WriteLine("predict sine function for 25 random values:");
			for(i=0;i<25;i++) {
				random_i_vec[0]=(decimal)r.NextDouble();
				prediction=tar.Predict(random_i_vec,m_i_vec);
				double denormalised_value=(double)random_i_vec[0]*2.0*Math.PI;
				double true_result=Math.Sin(denormalised_value);
				Console.WriteLine("sin({0:0.000}) = {1: 0.000;-0.000} (true result = {2: 0.000;-0.000})",denormalised_value,(prediction.d_vec_prediction[0]-0.5m)*2.0m,true_result);
			}
		}
	}
}